@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2000,2003,2005 by Solar Designer
+ * Copyright (c) 1996-2000,2003,2005,2011 by Solar Designer
  */
 
 #include <stdio.h>
@@ -14,6 +14,8 @@
 #include "memory.h"
 #include "compiler.h"
 
+#undef PRINT_INSNS
+
 char *c_errors[] = {
 	NULL,	/* No error */
 	"Unknown identifier",
@@ -107,8 +109,12 @@ struct c_op {
 #ifdef __GNUC__
 static struct c_op c_ops[];
 #else
+#ifdef PRINT_INSNS
+static struct c_op c_ops[52];
+#else
 static struct c_op c_ops[38];
 #endif
+#endif
 
 static void c_init(void)
 {
@@ -352,6 +358,12 @@ static void (*c_op_push_imm_imm)(void);
 static void (*c_op_push_imm_mem)(void);
 static void (*c_op_push_mem_imm)(void);
 static void (*c_op_push_mem_mem)(void);
+static void (*c_op_push_mem_mem_mem)(void);
+static void (*c_op_push_mem_mem_mem_imm)(void);
+static void (*c_op_push_mem_mem_mem_mem)(void);
+
+static void (*c_op_assign)(void);
+static void (*c_op_assign_pop)(void);
 
 static void (*c_push
 	(void (*last)(void), void (*op)(void), union c_insn *value))(void)
@@ -374,6 +386,23 @@ static void (*c_push
 			*c_code_ptr = *value;
 		}
 		c_code_ptr++;
+	} else if (last == c_op_push_mem_mem && op == c_op_push_mem) {
+		last = c_op_push_mem_mem_mem;
+		if (c_pass) {
+			(c_code_ptr - 3)->op = last;
+			*c_code_ptr = *value;
+		}
+		c_code_ptr++;
+	} else if (last == c_op_push_mem_mem_mem) {
+		if (op == c_op_push_imm)
+			last = c_op_push_mem_mem_mem_imm;
+		else
+			last = c_op_push_mem_mem_mem_mem;
+		if (c_pass) {
+			(c_code_ptr - 4)->op = last;
+			*c_code_ptr = *value;
+		}
+		c_code_ptr++;
 	} else {
 		last = op;
 
@@ -448,7 +477,7 @@ static int c_define(char term, struct c_ident **vars, struct c_ident *globals)
 	return c_errno;
 }
 
-static int c_expr(char term, struct c_ident *vars, char *token)
+static int c_expr(char term, struct c_ident *vars, char *token, int pop)
 {
 	char c;
 	struct c_ident *var;
@@ -573,9 +602,16 @@ static int c_expr(char term, struct c_ident *vars, char *token)
 
 	if (sp || balance) c_errno = C_ERROR_COUNT;
 
-	if (c_pass)
-		c_code_ptr->op = c_op_pop;
-	c_code_ptr++;
+	if (pop) {
+		if (last == c_op_assign) {
+			if (c_pass)
+				(c_code_ptr - 1)->op = c_op_assign_pop;
+		} else {
+			if (c_pass)
+				c_code_ptr->op = c_op_pop;
+			c_code_ptr++;
+		}
+	}
 
 	if (!term && !c_errno) c_errno = C_ERROR_NOTINFUNC;
 	if (*token == term) return -1;
@@ -596,7 +632,7 @@ static int c_cond(char term, struct c_ident *vars, char *token)
 	start = c_code_ptr;
 
 	if (c_expect('(')) return c_errno;
-	switch (c_expr(')', vars, c_gettoken())) {
+	switch (c_expr(')', vars, c_gettoken(), 0)) {
 	case -1:
 		break;
 
@@ -608,7 +644,8 @@ static int c_cond(char term, struct c_ident *vars, char *token)
 	}
 
 	if (c_pass)
-		(c_code_ptr - 1)->op = c_op_bz;
+		c_code_ptr->op = c_op_bz;
+	c_code_ptr++;
 	fixup = c_code_ptr++;
 
 	outer_loop_start = c_loop_start;
@@ -760,7 +797,7 @@ static int c_block(char term, struct c_ident *vars)
 		} else
 
 		if (*token != ';')
-			if (c_expr(term, locals, token)) break;
+			if (c_expr(term, locals, token, 1)) break;
 
 		if (c_errno) break;
 	}
@@ -772,14 +809,14 @@ static int c_block(char term, struct c_ident *vars)
 	return c_errno;
 }
 
-#ifdef __GNUC__
+#if defined(__GNUC__) && !defined(PRINT_INSNS)
 static void c_direct(union c_insn *addr);
 #endif
 
 int c_compile(int (*ext_getchar)(void), void (*ext_rewind)(void),
 	struct c_ident *externs)
 {
-#ifdef __GNUC__
+#if defined(__GNUC__) && !defined(PRINT_INSNS)
 	c_direct(NULL);
 #endif
 
@@ -792,6 +829,10 @@ int c_compile(int (*ext_getchar)(void), void (*ext_rewind)(void),
 	for (c_pass = 0; c_pass < 2; c_pass++) {
 		c_init();
 		c_block(0, externs);
+#ifdef PRINT_INSNS
+		fprintf(stderr, "Code size: %u\n",
+		    (unsigned int)(c_code_ptr - c_code_start));
+#endif
 
 		if (!c_pass) {
 			c_free_ident(c_funcs, NULL);
@@ -814,13 +855,9 @@ struct c_ident *c_lookup(char *name)
 
 void c_execute(struct c_ident *fn)
 {
-#ifndef __GNUC__
-	void (*op)(void);
-#endif
-
 	if (!fn) return;
 
-#ifdef __GNUC__
+#if defined(__GNUC__) && !defined(PRINT_INSNS)
 	c_direct(fn->addr);
 #else
 	c_stack[0].pc = NULL;
@@ -828,13 +865,19 @@ void c_execute(struct c_ident *fn)
 
 	c_pc = fn->addr;
 	do {
-		op = (c_pc++)->op;
+		void (*op)(void) = (c_pc++)->op;
+#ifdef PRINT_INSNS
+		int i = 0;
+		while (c_ops[i].op != op && c_ops[i].prec >= 0)
+			i++;
+		fprintf(stderr, "op: %s\n", c_ops[i].name);
+#endif
 		op();
 	} while (c_pc);
 #endif
 }
 
-#ifdef __GNUC__
+#if defined(__GNUC__) && !defined(PRINT_INSNS)
 
 static void c_direct(union c_insn *addr)
 {
@@ -879,7 +922,7 @@ static void c_direct(union c_insn *addr)
 		&&op_inc_l,
 		&&op_dec_l,
 		&&op_inc_r,
-		&&op_dec_r,
+		&&op_dec_r
 	};
 
 	if (!addr) {
@@ -896,6 +939,12 @@ static void c_direct(union c_insn *addr)
 		c_op_push_imm_mem = &&op_push_imm_mem;
 		c_op_push_mem_imm = &&op_push_mem_imm;
 		c_op_push_mem_mem = &&op_push_mem_mem;
+		c_op_push_mem_mem_mem = &&op_push_mem_mem_mem;
+		c_op_push_mem_mem_mem_imm = &&op_push_mem_mem_mem_imm;
+		c_op_push_mem_mem_mem_mem = &&op_push_mem_mem_mem_mem;
+
+		c_op_assign = &&op_assign;
+		c_op_assign_pop = &&op_assign_pop;
 
 		do {
 			c_ops[op].op = ops[op];
@@ -971,6 +1020,35 @@ op_push_mem_mem:
 	sp += 4;
 	goto *(pc - 1)->op;
 
+op_push_mem_mem_mem:
+	(sp - 2)->imm = imm;
+	sp->imm = *((sp + 1)->mem = pc->mem);
+	(sp + 2)->imm = *((sp + 3)->mem = (pc + 1)->mem);
+	imm = *((sp + 5)->mem = (pc + 2)->mem);
+	pc += 4;
+	sp += 6;
+	goto *(pc - 1)->op;
+
+op_push_mem_mem_mem_imm:
+	(sp - 2)->imm = imm;
+	sp->imm = *((sp + 1)->mem = pc->mem);
+	(sp + 2)->imm = *((sp + 3)->mem = (pc + 1)->mem);
+	(sp + 4)->imm = *((sp + 5)->mem = (pc + 2)->mem);
+	imm = (pc + 3)->imm;
+	pc += 5;
+	sp += 8;
+	goto *(pc - 1)->op;
+
+op_push_mem_mem_mem_mem:
+	(sp - 2)->imm = imm;
+	sp->imm = *((sp + 1)->mem = pc->mem);
+	(sp + 2)->imm = *((sp + 3)->mem = (pc + 1)->mem);
+	(sp + 4)->imm = *((sp + 5)->mem = (pc + 2)->mem);
+	imm = *((sp + 7)->mem = (pc + 3)->mem);
+	pc += 5;
+	sp += 8;
+	goto *(pc - 1)->op;
+
 op_index:
 	imm = *((sp - 3)->mem += imm);
 	sp -= 2;
@@ -981,6 +1059,11 @@ op_assign:
 	sp -= 2;
 	goto *(pc++)->op;
 
+op_assign_pop:
+	*(sp - 3)->mem = imm;
+	sp -= 4;
+	goto *(pc++)->op;
+
 op_add_a:
 	imm = *(sp - 3)->mem += imm;
 	sp -= 2;
@@ -1209,18 +1292,60 @@ static void c_f_op_push_mem_mem(void)
 	c_sp += 4;
 }
 
+static void c_f_op_push_mem_mem_mem(void)
+{
+	c_sp->imm = *c_pc->mem;
+	(c_sp + 1)->mem = (c_pc++)->mem;
+	(c_sp + 2)->imm = *c_pc->mem;
+	(c_sp + 3)->mem = (c_pc++)->mem;
+	(c_sp + 4)->imm = *c_pc->mem;
+	(c_sp + 5)->mem = (c_pc++)->mem;
+	c_sp += 6;
+}
+
+static void c_f_op_push_mem_mem_mem_imm(void)
+{
+	c_sp->imm = *c_pc->mem;
+	(c_sp + 1)->mem = (c_pc++)->mem;
+	(c_sp + 2)->imm = *c_pc->mem;
+	(c_sp + 3)->mem = (c_pc++)->mem;
+	(c_sp + 4)->imm = *c_pc->mem;
+	(c_sp + 5)->mem = (c_pc++)->mem;
+	(c_sp + 6)->imm = (c_pc++)->imm;
+	c_sp += 8;
+}
+
+static void c_f_op_push_mem_mem_mem_mem(void)
+{
+	c_sp->imm = *c_pc->mem;
+	(c_sp + 1)->mem = (c_pc++)->mem;
+	(c_sp + 2)->imm = *c_pc->mem;
+	(c_sp + 3)->mem = (c_pc++)->mem;
+	(c_sp + 4)->imm = *c_pc->mem;
+	(c_sp + 5)->mem = (c_pc++)->mem;
+	(c_sp + 6)->imm = *c_pc->mem;
+	(c_sp + 7)->mem = (c_pc++)->mem;
+	c_sp += 8;
+}
+
 static void c_op_index(void)
 {
 	c_sp -= 2;
 	(c_sp - 2)->imm = *((c_sp - 1)->mem += c_sp->imm);
 }
 
-static void c_op_assign(void)
+static void c_f_op_assign(void)
 {
 	c_sp -= 2;
 	(c_sp - 2)->imm = *(c_sp - 1)->mem = c_sp->imm;
 }
 
+static void c_f_op_assign_pop(void)
+{
+	c_sp -= 4;
+	*(c_sp + 1)->mem = (c_sp + 2)->imm;
+}
+
 static void c_op_add_a(void)
 {
 	c_sp -= 2;
@@ -1423,10 +1548,16 @@ static void (*c_op_push_imm_imm)(void) = c_f_op_push_imm_imm;
 static void (*c_op_push_imm_mem)(void) = c_f_op_push_imm_mem;
 static void (*c_op_push_mem_imm)(void) = c_f_op_push_mem_imm;
 static void (*c_op_push_mem_mem)(void) = c_f_op_push_mem_mem;
+static void (*c_op_push_mem_mem_mem)(void) = c_f_op_push_mem_mem_mem;
+static void (*c_op_push_mem_mem_mem_imm)(void) = c_f_op_push_mem_mem_mem_imm;
+static void (*c_op_push_mem_mem_mem_mem)(void) = c_f_op_push_mem_mem_mem_mem;
+
+static void (*c_op_assign)(void) = c_f_op_assign;
+static void (*c_op_assign_pop)(void) = c_f_op_assign_pop;
 
 static struct c_op c_ops[] = {
 	{1, C_LEFT_TO_RIGHT, C_CLASS_BINARY, "[", c_op_index},
-	{2, C_RIGHT_TO_LEFT, C_CLASS_BINARY, "=", c_op_assign},
+	{2, C_RIGHT_TO_LEFT, C_CLASS_BINARY, "=", c_f_op_assign},
 	{2, C_RIGHT_TO_LEFT, C_CLASS_BINARY, "+=", c_op_add_a},
 	{2, C_RIGHT_TO_LEFT, C_CLASS_BINARY, "-=", c_op_sub_a},
 	{2, C_RIGHT_TO_LEFT, C_CLASS_BINARY, "*=", c_op_mul_a},
@@ -1462,5 +1593,23 @@ static struct c_op c_ops[] = {
 	{12, C_LEFT_TO_RIGHT, C_CLASS_LEFT, "--", c_op_dec_l},
 	{12, C_LEFT_TO_RIGHT, C_CLASS_RIGHT, "++", c_op_inc_r},
 	{12, C_LEFT_TO_RIGHT, C_CLASS_RIGHT, "--", c_op_dec_r},
+#ifdef PRINT_INSNS
+	{0, 0, 0, "return", c_f_op_return},
+	{0, 0, 0, "bz", c_f_op_bz},
+	{0, 0, 0, "ba", c_f_op_ba},
+	{0, 0, 0, "push_imm", c_f_op_push_imm},
+	{0, 0, 0, "push_mem", c_f_op_push_mem},
+	{0, 0, 0, "pop", c_f_op_pop},
+	{0, 0, 0, "push_imm_imm", c_f_op_push_imm_imm},
+	{0, 0, 0, "push_imm_mem", c_f_op_push_imm_mem},
+	{0, 0, 0, "push_mem_imm", c_f_op_push_mem_imm},
+	{0, 0, 0, "push_mem_mem", c_f_op_push_mem_mem},
+	{0, 0, 0, "push_mem_mem_mem", c_f_op_push_mem_mem_mem},
+	{0, 0, 0, "push_mem_mem_mem_imm", c_f_op_push_mem_mem_mem_imm},
+	{0, 0, 0, "push_mem_mem_mem_mem", c_f_op_push_mem_mem_mem_mem},
+	{0, 0, 0, "assign_pop", c_f_op_assign_pop},
+	{-1}
+#else
 	{0}
+#endif
 };
