@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2000,2005 by Solar Designer
+ * Copyright (c) 1996-2000,2005,2008,2010 by Solar Designer
  */
 
 /*
@@ -32,11 +32,26 @@
 #define DES_SCALE			0
 #define DES_EXTB			1
 #define DES_COPY			0
-#define DES_BS_ASM			1
-#define DES_BS				1
-#define DES_BS_VECTOR			4
+#define DES_BS_ASM			0
+#define DES_BS				3
 #define DES_BS_EXPAND			0
+
+#if 1
+#define DES_BS_VECTOR			4
 #define DES_BS_ALGORITHM_NAME		"128/128 BS AltiVec"
+#elif 0
+/* It is likely unreasonable to use S-box expressions requiring vsel when this
+ * operation is only available in one of the two instruction sets.
+ * So let's revert to less demanding S-box expressions. */
+#undef DES_BS
+#define DES_BS				1
+#define DES_BS_VECTOR			5
+#define DES_BS_VECTOR_SIZE		8
+#define DES_BS_ALGORITHM_NAME		"128/128 BS AltiVec + 32/32 BS"
+#else
+#define DES_BS_VECTOR			8
+#define DES_BS_ALGORITHM_NAME		"128/128 X2 BS AltiVec"
+#endif
 
 #define MD5_ASM				0
 #define MD5_X2				1
@@ -44,5 +59,6 @@
 
 #define BF_ASM				0
 #define BF_SCALE			0
+#define BF_X2				0
 
 #endif
