@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2001,2005 by Solar Designer
+ * Copyright (c) 1996-2001,2005,2006,2010 by Solar Designer
  */
 
 /*
@@ -15,9 +15,11 @@
 #include "loader.h"
 
 /*
- * Crash recovery file name.
+ * Crash recovery file name and whether it has been "completed" (by adding
+ * the filename suffix to the session name).
  */
 extern char *rec_name;
+extern int rec_name_completed;
 
 /*
  * Crash recovery file format version number.
@@ -36,6 +38,11 @@ extern char **rec_argv;
  */
 extern unsigned int rec_check;
 
+/*
+ * Are we between a rec_restore_args() and a rec_restore_mode()?
+ */
+extern int rec_restoring_now;
+
 /*
  * Opens the crash recovery file for writing, and sets a function that will
  * be called to save cracking mode specific information.
@@ -51,7 +58,7 @@ extern void rec_save(void);
  * Closes the crash recovery file.
  * If the session is complete the file is unlinked.
  */
-extern void rec_done(int aborted);
+extern void rec_done(int save);
 
 /*
  * Opens the file and restores command line arguments. Leaves the file open.
